@@ -31,17 +31,19 @@ import java.util.concurrent.ExecutorService;
 import javax.inject.Inject;
 import javax.inject.Named;
 
+import cx.ring.daemon.Blob;
 import cx.ring.daemon.ConfigurationCallback;
-import cx.ring.daemon.IntVect;
 import cx.ring.daemon.Ringservice;
 import cx.ring.daemon.StringMap;
 import cx.ring.daemon.StringVect;
 import cx.ring.daemon.UintVect;
 import cx.ring.model.Account;
 import cx.ring.model.AccountConfig;
+import cx.ring.model.CallContact;
 import cx.ring.model.Codec;
 import cx.ring.model.ConfigKey;
-import cx.ring.model.DaemonEvent;
+import cx.ring.model.ServiceEvent;
+import cx.ring.model.TrustRequest;
 import cx.ring.model.Uri;
 import cx.ring.utils.FutureUtils;
 import cx.ring.utils.Log;
@@ -69,10 +71,12 @@ import ezvcard.VCard;
  * - EXPORT_ON_RING_ENDED
  * - NAME_REGISTRATION_ENDED
  * - REGISTERED_NAME_FOUND
+ * - MIGRATION_ENDED
+ * - INCOMING_TRUST_REQUEST
  */
 public class AccountService extends Observable {
 
-    private static final String TAG = AccountService.class.getName();
+    private static final String TAG = AccountService.class.getSimpleName();
 
     private static final int VCARD_CHUNK_SIZE = 1000;
 
@@ -89,7 +93,7 @@ public class AccountService extends Observable {
 
     private Account mCurrentAccount;
     private List<Account> mAccountList;
-    private ConfigurationCallback mCallbackHandler;
+    private ConfigurationCallbackHandler mCallbackHandler;
     private boolean mHasSipAccount;
     private boolean mHasRingAccount;
 
@@ -98,7 +102,7 @@ public class AccountService extends Observable {
         mAccountList = new ArrayList<>();
     }
 
-    public ConfigurationCallback getCallbackHandler() {
+    public ConfigurationCallbackHandler getCallbackHandler() {
         return mCallbackHandler;
     }
 
@@ -134,6 +138,10 @@ public class AccountService extends Observable {
 
                 setAccountsActive(isConnected);
                 Ringservice.connectivityChanged();
+
+                setChanged();
+                ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.ACCOUNTS_CHANGED);
+                notifyObservers(event);
             }
         });
     }
@@ -146,20 +154,47 @@ public class AccountService extends Observable {
             List<Map<String, String>> credentials = getCredentials(accountId);
             Map<String, String> volatileAccountDetails = getVolatileAccountDetails(accountId);
             Account account = new Account(accountId, details, credentials, volatileAccountDetails);
-            account.setDevices(getKnownRingDevices(accountId));
+            mAccountList.add(account);
 
             if (account.isSip()) {
                 mHasSipAccount = true;
-            }
-
-            if (account.isRing()) {
+            } else if (account.isRing()) {
                 mHasRingAccount = true;
-            }
 
-            mAccountList.add(account);
+                account.setDevices(getKnownRingDevices(accountId));
+                account.setContacts(getContacts(accountId));
+                List<Map<String, String>> requests = getTrustRequests(accountId);
+                for (Map<String, String> requestInfo : requests) {
+                    TrustRequest request = new TrustRequest(accountId, requestInfo);
+                    account.addRequest(request);
+                    // If name is in cache this can be synchronous
+                    lookupAddress(accountId, "", request.getContactId());
+                }
+            }
         }
     }
 
+    private Account getAccountByName(final String name) {
+        for (Account acc : mAccountList) {
+            if (acc.getAlias().equals(name))
+                return acc;
+        }
+        return null;
+    }
+
+    public String getNewAccountName(final String prefix) {
+        String name = String.format(prefix, "").trim();
+        if (getAccountByName(name) == null) {
+            return name;
+        }
+        int num = 1;
+        do {
+            num++;
+            name = String.format(prefix, num).trim();
+        } while (getAccountByName(name) != null);
+        return name;
+    }
+
     /**
      * Adds a new Account in the Daemon (also sends an ACCOUNT_ADDED event)
      * Sets the new account as the current one
@@ -204,9 +239,9 @@ public class AccountService extends Observable {
         }
 
         setChanged();
-        DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.ACCOUNT_ADDED);
-        event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-        event.addEventInput(DaemonEvent.EventInput.STATE, account.getRegistrationState());
+        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.ACCOUNT_ADDED);
+        event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+        event.addEventInput(ServiceEvent.EventInput.STATE, account.getRegistrationState());
         notifyObservers(event);
 
         setCurrentAccount(account);
@@ -278,8 +313,7 @@ public class AccountService extends Observable {
 
                 VCard vcard = VCardUtils.loadLocalProfileFromDisk(
                         mDeviceRuntimeService.provideFilesDir(),
-                        accountId,
-                        mDeviceRuntimeService.provideDefaultVCardName());
+                        accountId);
                 String stringVCard = VCardUtils.vcardToString(vcard);
 
                 int nbTotal = stringVCard.length() / VCARD_CHUNK_SIZE + (stringVCard.length() % VCARD_CHUNK_SIZE != 0 ? 1 : 0);
@@ -494,7 +528,7 @@ public class AccountService extends Observable {
         }
 
         setChanged();
-        DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.ACCOUNTS_CHANGED);
+        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.ACCOUNTS_CHANGED);
         notifyObservers(event);
     }
 
@@ -565,7 +599,7 @@ public class AccountService extends Observable {
                 new Callable<String>() {
                     @Override
                     public String call() throws Exception {
-                        Log.i(TAG, "addRingDevice() thread running...");
+                        Log.i(TAG, "exportOnRing() thread running...");
                         return Ringservice.exportOnRing(accountId, password);
                     }
                 }
@@ -592,6 +626,50 @@ public class AccountService extends Observable {
         );
     }
 
+    /**
+     * @param accountId id of the account used with the device
+     * @param deviceId  id of the device to revoke
+     * @param password  password of the account
+     */
+    public void revokeDevice(final String accountId, final String password, final String deviceId) {
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "revokeDevice() thread running...");
+                        return Ringservice.revokeDevice(accountId, password, deviceId);
+                    }
+                }
+        );
+    }
+
+    /**
+     * @param accountId id of the account used with the device
+     * @param newName new device name
+     */
+    public void renameDevice(final String accountId, final String newName) {
+        final Account account = getAccount(accountId);
+        account.setDevices(FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                true,
+                new Callable<Map<String, String>>() {
+                    @Override
+                    public Map<String, String> call() throws Exception {
+                        Log.i(TAG, "renameDevice() thread running... " + newName);
+                        StringMap details = Ringservice.getAccountDetails(accountId);
+                        details.set(ConfigKey.ACCOUNT_DEVICE_NAME.key(), newName);
+                        Ringservice.setAccountDetails(accountId, details);
+                        return Ringservice.getKnownRingDevices(accountId).toNative();
+                    }
+                }
+        ));
+        account.setDetail(ConfigKey.ACCOUNT_DEVICE_NAME, newName);
+    }
+
     /**
      * Sets the active codecs list of the account in the Daemon
      *
@@ -870,54 +948,6 @@ public class AccountService extends Observable {
         );
     }
 
-    /**
-     * Looks up for the availibility of the name on the blockchain
-     *
-     * @param account
-     * @param nameserver
-     * @param name
-     */
-    public void lookupName(final String account, final String nameserver, final String name) {
-
-        FutureUtils.executeDaemonThreadCallable(
-                mExecutor,
-                mDeviceRuntimeService.provideDaemonThreadId(),
-                false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "lookupName() thread running...");
-                        Ringservice.lookupName(account, nameserver, name);
-                        return true;
-                    }
-                }
-        );
-    }
-
-    /**
-     * Reverse looks up the address in the blockchain to find the name
-     *
-     * @param account
-     * @param nameserver
-     * @param address
-     */
-    public void lookupAddress(final String account, final String nameserver, final String address) {
-
-        FutureUtils.executeDaemonThreadCallable(
-                mExecutor,
-                mDeviceRuntimeService.provideDaemonThreadId(),
-                false,
-                new Callable<Boolean>() {
-                    @Override
-                    public Boolean call() throws Exception {
-                        Log.i(TAG, "lookupAddress() thread running...");
-                        Ringservice.lookupAddress(account, nameserver, address);
-                        return true;
-                    }
-                }
-        );
-    }
-
     /**
      * Registers a new name on the blockchain for the account
      *
@@ -959,7 +989,229 @@ public class AccountService extends Observable {
         );
     }
 
-    private class ConfigurationCallbackHandler extends ConfigurationCallback {
+    /* contact requests */
+
+    /**
+     * @param accountId
+     * @return all trust requests from the daemon for the account Id
+     */
+    public List<Map<String, String>> getTrustRequests(final String accountId) {
+
+        return FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                true,
+                new Callable<List<Map<String, String>>>() {
+                    @Override
+                    public List<Map<String, String>> call() throws Exception {
+                        Log.i(TAG, "getTrustRequests() thread running...");
+                        return Ringservice.getTrustRequests(accountId).toNative();
+                    }
+                }
+        );
+    }
+
+    /**
+     * Accepts a pending trust request
+     *
+     * @param accountId
+     * @param from
+     * @return
+     */
+    public Boolean acceptTrustRequest(final String accountId, final String from) {
+        Account account = getAccount(accountId);
+        account.removeRequest(from);
+        return FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "acceptTrustRequest() thread running...");
+                        boolean ok = Ringservice.acceptTrustRequest(accountId, from);
+                        if (ok) {
+                            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.INCOMING_TRUST_REQUEST);
+                            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+                            notifyObservers(event);
+                        }
+                        return ok;
+                    }
+                }
+        );
+    }
+
+    /**
+     * Refuses and blocks a pending trust request
+     *
+     * @param accountId
+     * @param from
+     * @return
+     */
+    public void discardTrustRequest(final String accountId, final String from) {
+        Account account = getAccount(accountId);
+        account.removeRequest(from);
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Void>() {
+                    @Override
+                    public Void call() throws Exception {
+                        Log.i(TAG, "discardTrustRequest() " + accountId + " " + from);
+                        boolean ok = Ringservice.discardTrustRequest(accountId, from);
+                        if (ok) {
+                            setChanged();
+                            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.INCOMING_TRUST_REQUEST);
+                            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+                            notifyObservers(event);
+                        }
+                        return null;
+                    }
+                }
+        );
+    }
+
+    /**
+     * Sends a new trust request
+     *
+     * @param accountId
+     * @param to
+     * @param message
+     */
+    public void sendTrustRequest(final String accountId, final String to, final Blob message) {
+
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "sendTrustRequest() thread running...");
+                        Ringservice.sendTrustRequest(accountId, to, message);
+                        return true;
+                    }
+                }
+        );
+    }
+
+
+    /**
+     * Add a new contact for the account Id on the Daemon
+     *
+     * @param accountId
+     * @param uri
+     */
+    public void addContact(final String accountId, final String uri) {
+
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "addContact() thread running...");
+                        Ringservice.addContact(accountId, uri);
+                        return true;
+                    }
+                }
+        );
+    }
+
+    /**
+     * Remove an existing contact for the account Id on the Daemon
+     *
+     * @param accountId
+     * @param uri
+     */
+    public void removeContact(final String accountId, final String uri, final boolean ban) {
+
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "removeContact() thread running...");
+                        Ringservice.removeContact(accountId, uri, ban);
+                        return true;
+                    }
+                }
+        );
+    }
+
+    /**
+     * @param accountId
+     * @return the contacts list from the daemon
+     */
+    public List<Map<String, String>> getContacts(final String accountId) {
+
+        return FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                true,
+                new Callable<List<Map<String, String>>>() {
+                    @Override
+                    public List<Map<String, String>> call() throws Exception {
+                        Log.i(TAG, "getContacts() thread running...");
+                        return Ringservice.getContacts(accountId).toNative();
+                    }
+                }
+        );
+    }
+
+    /**
+     * Looks up for the availibility of the name on the blockchain
+     *
+     * @param account
+     * @param nameserver
+     * @param name
+     */
+    public void lookupName(final String account, final String nameserver, final String name) {
+
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "lookupName() thread running...");
+                        Ringservice.lookupName(account, nameserver, name);
+                        return true;
+                    }
+                }
+        );
+    }
+
+    /**
+     * Reverse looks up the address in the blockchain to find the name
+     *
+     * @param account
+     * @param nameserver
+     * @param address
+     */
+    public void lookupAddress(final String account, final String nameserver, final String address) {
+
+        FutureUtils.executeDaemonThreadCallable(
+                mExecutor,
+                mDeviceRuntimeService.provideDaemonThreadId(),
+                false,
+                new Callable<Boolean>() {
+                    @Override
+                    public Boolean call() throws Exception {
+                        Log.i(TAG, "lookupAddress() " + address);
+                        Ringservice.lookupAddress(account, nameserver, address);
+                        return true;
+                    }
+                }
+        );
+    }
+
+    class ConfigurationCallbackHandler extends ConfigurationCallback {
 
         @Override
         public void volumeChanged(String device, int value) {
@@ -967,9 +1219,9 @@ public class AccountService extends Observable {
             super.volumeChanged(device, value);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.VOLUME_CHANGED);
-            event.addEventInput(DaemonEvent.EventInput.DEVICE, device);
-            event.addEventInput(DaemonEvent.EventInput.VALUE, value);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.VOLUME_CHANGED);
+            event.addEventInput(ServiceEvent.EventInput.DEVICE, device);
+            event.addEventInput(ServiceEvent.EventInput.VALUE, value);
             notifyObservers(event);
         }
 
@@ -990,7 +1242,7 @@ public class AccountService extends Observable {
             Account currentAccount = getAccount(currentAccountId);
             if (currentAccount != null) {
                 mCurrentAccount = currentAccount;
-            } else if (!mAccountList.isEmpty()){
+            } else if (!mAccountList.isEmpty()) {
                 // no current account, by default it will be the first one
                 mCurrentAccount = mAccountList.get(0);
             } else {
@@ -998,7 +1250,7 @@ public class AccountService extends Observable {
             }
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.ACCOUNTS_CHANGED);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.ACCOUNTS_CHANGED);
             notifyObservers(event);
         }
 
@@ -1007,8 +1259,8 @@ public class AccountService extends Observable {
             Log.d(TAG, "stun status failure: " + accountId);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.STUN_STATUS_FAILURE);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.STUN_STATUS_FAILURE);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
             notifyObservers(event);
         }
 
@@ -1033,11 +1285,11 @@ public class AccountService extends Observable {
 
             if (!oldState.equals(newState)) {
                 setChanged();
-                DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.REGISTRATION_STATE_CHANGED);
-                event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-                event.addEventInput(DaemonEvent.EventInput.STATE, newState);
-                event.addEventInput(DaemonEvent.EventInput.DETAIL_CODE, code);
-                event.addEventInput(DaemonEvent.EventInput.DETAIL_STRING, detailString);
+                ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.REGISTRATION_STATE_CHANGED);
+                event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+                event.addEventInput(ServiceEvent.EventInput.STATE, newState);
+                event.addEventInput(ServiceEvent.EventInput.DETAIL_CODE, code);
+                event.addEventInput(ServiceEvent.EventInput.DETAIL_STRING, detailString);
                 notifyObservers(event);
             }
         }
@@ -1057,10 +1309,10 @@ public class AccountService extends Observable {
             Log.d(TAG, "incomingAccountMessage: " + accountId + ", " + from + ", " + msg);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.INCOMING_ACCOUNT_MESSAGE);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-            event.addEventInput(DaemonEvent.EventInput.FROM, from);
-            event.addEventInput(DaemonEvent.EventInput.MESSAGES, msg);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.INCOMING_ACCOUNT_MESSAGE);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.FROM, from);
+            event.addEventInput(ServiceEvent.EventInput.MESSAGES, msg);
             notifyObservers(event);
         }
 
@@ -1069,11 +1321,11 @@ public class AccountService extends Observable {
             Log.d(TAG, "accountMessageStatusChanged: " + accountId + ", " + messageId + ", " + to + ", " + status);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.ACCOUNT_MESSAGE_STATUS_CHANGED);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-            event.addEventInput(DaemonEvent.EventInput.MESSAGE_ID, messageId);
-            event.addEventInput(DaemonEvent.EventInput.TO, to);
-            event.addEventInput(DaemonEvent.EventInput.STATE, status);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.ACCOUNT_MESSAGE_STATUS_CHANGED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.MESSAGE_ID, messageId);
+            event.addEventInput(ServiceEvent.EventInput.TO, to);
+            event.addEventInput(ServiceEvent.EventInput.STATE, status);
             notifyObservers(event);
         }
 
@@ -1082,29 +1334,8 @@ public class AccountService extends Observable {
             Log.d(TAG, "errorAlert : " + alert);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.ERROR_ALERT);
-            event.addEventInput(DaemonEvent.EventInput.ALERT, alert);
-            notifyObservers(event);
-        }
-
-        @Override
-        public void getHardwareAudioFormat(IntVect ret) {
-            Log.d(TAG, "getHardwareAudioFormat: " + ret.toString());
-
-            setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.GET_HARDWARE_AUDIO_FORMAT);
-            event.addEventInput(DaemonEvent.EventInput.FORMATS, ret);
-            notifyObservers(event);
-        }
-
-        @Override
-        public void getAppDataPath(String name, StringVect ret) {
-            Log.d(TAG, "getAppDataPath: " + name + ", " + ret);
-
-            setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.GET_APP_DATA_PATH);
-            event.addEventInput(DaemonEvent.EventInput.NAME, name);
-            event.addEventInput(DaemonEvent.EventInput.PATHS, ret);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.ERROR_ALERT);
+            event.addEventInput(ServiceEvent.EventInput.ALERT, alert);
             notifyObservers(event);
         }
 
@@ -1116,9 +1347,9 @@ public class AccountService extends Observable {
             accountChanged.setDevices(devices.toNative());
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.KNOWN_DEVICES_CHANGED);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-            event.addEventInput(DaemonEvent.EventInput.DEVICES, devices);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.KNOWN_DEVICES_CHANGED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.DEVICES, devices);
             notifyObservers(event);
         }
 
@@ -1127,10 +1358,10 @@ public class AccountService extends Observable {
             Log.d(TAG, "exportOnRingEnded: " + accountId + ", " + code + ", " + pin);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.EXPORT_ON_RING_ENDED);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-            event.addEventInput(DaemonEvent.EventInput.CODE, code);
-            event.addEventInput(DaemonEvent.EventInput.PIN, pin);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.EXPORT_ON_RING_ENDED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.CODE, code);
+            event.addEventInput(ServiceEvent.EventInput.PIN, pin);
             notifyObservers(event);
         }
 
@@ -1149,10 +1380,68 @@ public class AccountService extends Observable {
             acc.setDetail(ConfigKey.ACCOUNT_REGISTERED_NAME, name);
 
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.NAME_REGISTRATION_ENDED);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-            event.addEventInput(DaemonEvent.EventInput.STATE, state);
-            event.addEventInput(DaemonEvent.EventInput.NAME, name);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.NAME_REGISTRATION_ENDED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.STATE, state);
+            event.addEventInput(ServiceEvent.EventInput.NAME, name);
+            notifyObservers(event);
+        }
+
+        @Override
+        public void migrationEnded(String accountId, String state) {
+            Log.d(TAG, "migrationEnded: " + accountId + ", " + state);
+
+            setChanged();
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.MIGRATION_ENDED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.STATE, state);
+            notifyObservers(event);
+        }
+
+        @Override
+        public void deviceRevocationEnded(String accountId, String device, int state) {
+            Log.d(TAG, "deviceRevocationEnded: " + accountId + ", " + device + ", " + state);
+
+            setChanged();
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.DEVICE_REVOCATION_ENDED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.DEVICE, device);
+            event.addEventInput(ServiceEvent.EventInput.STATE, state);
+            notifyObservers(event);
+        }
+
+        @Override
+        public void incomingTrustRequest(String accountId, String from, Blob message, long received) {
+            Log.d(TAG, "incomingTrustRequest: " + accountId + ", " + from + ", " + message + ", " + received);
+
+            Account account = getAccount(accountId);
+            if (account != null) {
+                TrustRequest request = new TrustRequest(accountId, from, received, message.toJavaString());
+                account.addRequest(request);
+                lookupAddress(accountId, "", from);
+            }
+        }
+
+
+        @Override
+        public void contactAdded(String accountId, String uri, boolean confirmed) {
+            Log.d(TAG, "contactAdded: " + accountId + ", " + uri + ", " + confirmed);
+
+            setChanged();
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.CONTACT_ADDED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.CONFIRMED, confirmed);
+            notifyObservers(event);
+        }
+
+        @Override
+        public void contactRemoved(String accountId, String uri, boolean banned) {
+            Log.d(TAG, "contactRemoved: " + accountId + ", " + uri + ", " + banned);
+
+            setChanged();
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.CONTACT_REMOVED);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.BANNED, banned);
             notifyObservers(event);
         }
 
@@ -1160,13 +1449,40 @@ public class AccountService extends Observable {
         public void registeredNameFound(String accountId, int state, String address, String name) {
             Log.d(TAG, "registeredNameFound: " + accountId + ", " + state + ", " + name + ", " + address);
 
+            Account account = getAccount(accountId);
+            if (account != null) {
+                if (state == 0) {
+                    CallContact contact = account.getContact(address);
+                    if (contact != null) {
+                        contact.setUsername(name);
+                    }
+                }
+                TrustRequest request = account.getRequest(address);
+                if (request != null) {
+                    Log.d(TAG, "registeredNameFound: updating TrustRequest " +  name);
+                    boolean resolved = request.isNameResolved();
+                    request.setUsername(name);
+                    if (!resolved) {
+                        Log.d(TAG, "registeredNameFound: TrustRequest resolved " +  name);
+                        setChanged();
+                        ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.INCOMING_TRUST_REQUEST);
+                        event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+                        event.addEventInput(ServiceEvent.EventInput.FROM, request.getContactId());
+                        notifyObservers(event);
+                    }
+                } else {
+                    Log.d(TAG, "registeredNameFound: no TrustRequest for " +  address);
+                }
+            }
+
             setChanged();
-            DaemonEvent event = new DaemonEvent(DaemonEvent.EventType.REGISTERED_NAME_FOUND);
-            event.addEventInput(DaemonEvent.EventInput.ACCOUNT_ID, accountId);
-            event.addEventInput(DaemonEvent.EventInput.STATE, state);
-            event.addEventInput(DaemonEvent.EventInput.ADDRESS, address);
-            event.addEventInput(DaemonEvent.EventInput.NAME, name);
+            ServiceEvent event = new ServiceEvent(ServiceEvent.EventType.REGISTERED_NAME_FOUND);
+            event.addEventInput(ServiceEvent.EventInput.ACCOUNT_ID, accountId);
+            event.addEventInput(ServiceEvent.EventInput.STATE, state);
+            event.addEventInput(ServiceEvent.EventInput.ADDRESS, address);
+            event.addEventInput(ServiceEvent.EventInput.NAME, name);
             notifyObservers(event);
         }
+
     }
 }
\ No newline at end of file
