@@ -81,7 +81,6 @@ struct _AccountCreationWizardPrivate
 
     /* account creation */
     GtkWidget *account_creation;
-    GtkWidget *vbox_account_creation_entry;
     GtkWidget *entry_username;
     GtkWidget *entry_password;
     GtkWidget *entry_password_confirm;
@@ -183,7 +182,6 @@ account_creation_wizard_class_init(AccountCreationWizardClass *klass)
 
     /* account creation */
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountCreationWizard, account_creation);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountCreationWizard, vbox_account_creation_entry);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountCreationWizard, entry_password);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountCreationWizard, entry_password_confirm);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS (klass), AccountCreationWizard, button_account_creation_next);
@@ -400,6 +398,13 @@ create_new_ring_account(AccountCreationWizard *win)
     g_return_val_if_fail(IS_ACCOUNT_CREATION_WIZARD(win), G_SOURCE_REMOVE);
     AccountCreationWizardPrivate *priv = ACCOUNT_CREATION_WIZARD_GET_PRIVATE(win);
 
+    /* Tuleap: #1441
+     * if the user did not validate the avatar area selection, we still take that as the image
+     * for their avatar; otherwise many users end up with no avatar by default
+     * TODO: improve avatar creation process to not need this fix
+     */
+    avatar_manipulation_wizard_completed(AVATAR_MANIPULATION(priv->avatar_manipulation));
+
     gchar *display_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(priv->entry_display_name)));
     gchar *username = g_strdup(gtk_entry_get_text(GTK_ENTRY(priv->entry_username)));
     gchar *password = g_strdup(gtk_entry_get_text(GTK_ENTRY(priv->entry_password)));
@@ -641,8 +646,6 @@ build_creation_wizard_view(AccountCreationWizard *view, gboolean show_cancel_but
     else
         gtk_entry_set_text(GTK_ENTRY(priv->entry_display_name), user_name);
 
-    gtk_entry_set_placeholder_text(GTK_ENTRY(priv->entry_username), user_name);
-
     /* cancel button */
     gtk_widget_set_visible(priv->button_wizard_cancel, show_cancel_button);
 
