@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2013-2017 Savoir-faire Linux Inc.
  *
  *  Author: Guillaume Roguez <Guillaume.Roguez@savoirfairelinux.com>
  *
@@ -26,6 +26,7 @@
 #include "audio/audiobuffer.h"
 #include "audio/ringbuffer.h"
 #include "audio/resampler.h"
+#include "video/decoder_finder.h"
 #include "manager.h"
 
 #ifdef RING_ACCEL
@@ -38,10 +39,14 @@
 #include <iostream>
 #include <unistd.h>
 #include <thread> // hardware_concurrency
+#include <chrono>
 
 namespace ring {
 
-using std::string;
+// maximum number of packets the jitter buffer can queue
+const unsigned jitterBufferMaxSize_ {1500};
+// maximum time a packet can be queued
+const constexpr auto jitterBufferMaxDelay_ = std::chrono::milliseconds(50);
 
 MediaDecoder::MediaDecoder() :
     inputCtx_(avformat_alloc_context()),
@@ -89,7 +94,8 @@ int MediaDecoder::openInput(const DeviceParams& params)
 
     // Set jitter buffer options
     av_dict_set(&options_, "reorder_queue_size",ring::to_string(jitterBufferMaxSize_).c_str(), 0);
-    av_dict_set(&options_, "max_delay",ring::to_string(jitterBufferMaxDelay_).c_str(), 0);
+    auto us = std::chrono::duration_cast<std::chrono::microseconds>(jitterBufferMaxDelay_).count();
+    av_dict_set(&options_, "max_delay",ring::to_string(us).c_str(), 0);
 
     if(!params.pixel_format.empty()){
         av_dict_set(&options_, "pixel_format", params.pixel_format.c_str(), 0);
@@ -146,11 +152,7 @@ int MediaDecoder::setupFromAudioData(const AudioFormat format)
     inputCtx_->max_analyze_duration = MAX_ANALYZE_DURATION * AV_TIME_BASE;
 
     RING_DBG("Finding stream info");
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 8, 0)
-    ret = av_find_stream_info(inputCtx_);
-#else
     ret = avformat_find_stream_info(inputCtx_, NULL);
-#endif
     RING_DBG("Finding stream info DONE");
 
     if (ret < 0) {
@@ -170,7 +172,11 @@ int MediaDecoder::setupFromAudioData(const AudioFormat format)
 
     // find the first audio stream from the input
     for (size_t i = 0; streamIndex_ == -1 && i < inputCtx_->nb_streams; ++i)
+#ifndef _WIN32
+        if (inputCtx_->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_AUDIO)
+#else
         if (inputCtx_->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
+#endif
             streamIndex_ = i;
 
     if (streamIndex_ == -1) {
@@ -180,6 +186,16 @@ int MediaDecoder::setupFromAudioData(const AudioFormat format)
 
     // Get a pointer to the codec context for the video stream
     avStream_ = inputCtx_->streams[streamIndex_];
+#ifndef _WIN32
+    inputDecoder_ = avcodec_find_decoder(avStream_->codecpar->codec_id);
+    if (!inputDecoder_) {
+        RING_ERR("Unsupported codec");
+        return -1;
+    }
+
+    decoderCtx_ = avcodec_alloc_context3(inputDecoder_);
+    avcodec_parameters_to_context(decoderCtx_, avStream_->codecpar);
+#else
     decoderCtx_ = avStream_->codec;
     if (decoderCtx_ == 0) {
         RING_ERR("Decoder context is NULL");
@@ -192,6 +208,7 @@ int MediaDecoder::setupFromAudioData(const AudioFormat format)
         RING_ERR("Unsupported codec");
         return -1;
     }
+#endif
 
     decoderCtx_->thread_count = std::thread::hardware_concurrency();
     decoderCtx_->channels = format.nb_channels;
@@ -205,9 +222,7 @@ int MediaDecoder::setupFromAudioData(const AudioFormat format)
         startTime_ = av_gettime();
     }
 
-#if LIBAVCODEC_VERSION_MAJOR >= 55
     decoderCtx_->refcounted_frames = 1;
-#endif
     ret = avcodec_open2(decoderCtx_, inputDecoder_, NULL);
     if (ret) {
         RING_ERR("Could not open codec");
@@ -231,11 +246,7 @@ int MediaDecoder::setupFromVideoData()
     inputCtx_->max_analyze_duration = MAX_ANALYZE_DURATION * AV_TIME_BASE;
 
     RING_DBG("Finding stream info");
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 8, 0)
-    ret = av_find_stream_info(inputCtx_);
-#else
     ret = avformat_find_stream_info(inputCtx_, NULL);
-#endif
     if (ret < 0) {
         // workaround for this bug:
         // http://patches.libav.org/patch/22541/
@@ -253,7 +264,11 @@ int MediaDecoder::setupFromVideoData()
 
     // find the first video stream from the input
     for (size_t i = 0; streamIndex_ == -1 && i < inputCtx_->nb_streams; ++i)
+#ifndef _WIN32
+        if (inputCtx_->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO)
+#else
         if (inputCtx_->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
+#endif
             streamIndex_ = i;
 
     if (streamIndex_ == -1) {
@@ -263,41 +278,49 @@ int MediaDecoder::setupFromVideoData()
 
     // Get a pointer to the codec context for the video stream
     avStream_ = inputCtx_->streams[streamIndex_];
+#ifndef _WIN32
+    inputDecoder_ = video::findDecoder(avStream_->codecpar->codec_id);
+    if (!inputDecoder_) {
+        RING_ERR("Unsupported codec");
+        return -1;
+    }
+
+    decoderCtx_ = avcodec_alloc_context3(inputDecoder_);
+    avcodec_parameters_to_context(decoderCtx_, avStream_->codecpar);
+#else
     decoderCtx_ = avStream_->codec;
     if (decoderCtx_ == 0) {
         RING_ERR("Decoder context is NULL");
         return -1;
     }
 
-    decoderCtx_->thread_count = std::thread::hardware_concurrency();
-
-#ifdef RING_ACCEL
-    if (enableAccel_) {
-        accel_ = video::makeHardwareAccel(decoderCtx_);
-        decoderCtx_->opaque = accel_.get();
-    }
-#endif // RING_ACCEL
-
     // find the decoder for the video stream
     inputDecoder_ = avcodec_find_decoder(decoderCtx_->codec_id);
     if (!inputDecoder_) {
         RING_ERR("Unsupported codec");
         return -1;
     }
+#endif
+    RING_DBG("Decoding video using %s (%s)", inputDecoder_->long_name, inputDecoder_->name);
+
+    decoderCtx_->thread_count = std::thread::hardware_concurrency();
+
+#ifdef RING_ACCEL
+    if (enableAccel_) {
+        accel_ = video::makeHardwareAccel(decoderCtx_);
+        decoderCtx_->opaque = accel_.get();
+    } else {
+        RING_WARN("Hardware accelerated decoding disabled by user preference");
+    }
+#endif // RING_ACCEL
 
     if (emulateRate_) {
         RING_DBG("Using framerate emulation");
         startTime_ = av_gettime();
     }
 
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(53, 6, 0)
-    ret = avcodec_open(decoderCtx_, inputDecoder_);
-#else
-#   if LIBAVCODEC_VERSION_MAJOR >= 55
     decoderCtx_->refcounted_frames = 1;
-#   endif
     ret = avcodec_open2(decoderCtx_, inputDecoder_, NULL);
-#endif
     if (ret) {
         RING_ERR("Could not open codec");
         return -1;
@@ -331,25 +354,39 @@ MediaDecoder::decode(VideoFrame& result)
 
     auto frame = result.pointer();
     int frameFinished = 0;
-    int len = avcodec_decode_video2(decoderCtx_, frame,
-                                    &frameFinished, &inpacket);
-    av_packet_unref(&inpacket);
-
-    if (len <= 0)
+    ret = avcodec_send_packet(decoderCtx_, &inpacket);
+    if (ret < 0) {
+#ifdef RING_ACCEL
+        if (accel_ && accel_->hasFailed())
+            return Status::RestartRequired;
+#endif
+        return ret == AVERROR_EOF ? Status::Success : Status::DecodeError;
+    }
+    ret = avcodec_receive_frame(decoderCtx_, frame);
+    if (ret < 0 && ret != AVERROR(EAGAIN) && ret != AVERROR_EOF) {
+#ifdef RING_ACCEL
+        if (accel_ && accel_->hasFailed())
+            return Status::RestartRequired;
+#endif
         return Status::DecodeError;
+    }
+    if (ret >= 0)
+        frameFinished = 1;
+
+    av_packet_unref(&inpacket);
 
     if (frameFinished) {
         frame->format = (AVPixelFormat) correctPixFmt(frame->format);
 #ifdef RING_ACCEL
         if (accel_) {
             if (!accel_->hasFailed())
-                accel_->extractData(decoderCtx_, result);
+                accel_->extractData(result);
             else
                 return Status::RestartRequired;
         }
 #endif // RING_ACCEL
-        if (emulateRate_ and frame->pkt_pts != AV_NOPTS_VALUE) {
-            auto frame_time = getTimeBase()*(frame->pkt_pts - avStream_->start_time);
+        if (emulateRate_ and frame->pts != AV_NOPTS_VALUE) {
+            auto frame_time = getTimeBase()*(frame->pts - avStream_->start_time);
             auto target = startTime_ + static_cast<std::int64_t>(frame_time.real() * 1e6);
             auto now = av_gettime();
             if (target > now) {
@@ -390,17 +427,20 @@ MediaDecoder::decode(const AudioFrame& decodedFrame)
     }
 
     int frameFinished = 0;
-    int len = avcodec_decode_audio4(decoderCtx_, frame,
-                                    &frameFinished, &inpacket);
-    av_packet_unref(&inpacket);
+        ret = avcodec_send_packet(decoderCtx_, &inpacket);
+        if (ret < 0)
+            return ret == AVERROR_EOF ? Status::Success : Status::DecodeError;
 
-    if (len <= 0) {
+    ret = avcodec_receive_frame(decoderCtx_, frame);
+    if (ret < 0 && ret != AVERROR(EAGAIN) && ret != AVERROR_EOF)
         return Status::DecodeError;
-    }
+    if (ret >= 0)
+        frameFinished = 1;
 
     if (frameFinished) {
-        if (emulateRate_ and frame->pkt_pts != AV_NOPTS_VALUE) {
-            auto frame_time = getTimeBase()*(frame->pkt_pts - avStream_->start_time);
+        av_packet_unref(&inpacket);
+        if (emulateRate_ and frame->pts != AV_NOPTS_VALUE) {
+            auto frame_time = getTimeBase()*(frame->pts - avStream_->start_time);
             auto target = startTime_ + static_cast<std::int64_t>(frame_time.real() * 1e6);
             auto now = av_gettime();
             if (target > now) {
@@ -421,20 +461,24 @@ MediaDecoder::flush(VideoFrame& result)
     av_init_packet(&inpacket);
 
     int frameFinished = 0;
-    auto len = avcodec_decode_video2(decoderCtx_, result.pointer(),
-                                    &frameFinished, &inpacket);
+    int ret = 0;
+    ret = avcodec_send_packet(decoderCtx_, &inpacket);
+    if (ret < 0)
+        return ret == AVERROR_EOF ? Status::Success : Status::DecodeError;
 
-    av_packet_unref(&inpacket);
-
-    if (len <= 0)
+    ret = avcodec_receive_frame(decoderCtx_, result.pointer());
+    if (ret < 0 && ret != AVERROR(EAGAIN) && ret != AVERROR_EOF)
         return Status::DecodeError;
+    if (ret >= 0)
+        frameFinished = 1;
 
     if (frameFinished) {
+        av_packet_unref(&inpacket);
 #ifdef RING_ACCEL
         // flush is called when closing the stream
         // so don't restart the media decoder
         if (accel_ && !accel_->hasFailed())
-            accel_->extractData(decoderCtx_, result);
+            accel_->extractData(result);
 #endif // RING_ACCEL
         return Status::FrameFinished;
     }
