@@ -40,6 +40,7 @@
 #include "dring/call_const.h"
 #include "dring/media_const.h"
 #include "client/ring_signal.h"
+#include "ice_transport.h"
 
 #ifdef RING_VIDEO
 #include "client/videomanager.h"
@@ -119,6 +120,8 @@ SIPCall::SIPCall(SIPAccountBase& account, const std::string& id, Call::CallType
 {
     if (account.getUPnPActive())
         upnp_.reset(new upnp::Controller());
+
+    setCallMediaLocal();
 }
 
 SIPCall::~SIPCall()
@@ -134,13 +137,11 @@ SIPCall::getSIPAccount() const
 }
 
 void
-SIPCall::setCallMediaLocal(const pj_sockaddr& localIP)
+SIPCall::setCallMediaLocal()
 {
-    setLocalIp(localIP);
-
-    if (getLocalAudioPort() == 0
+    if (localAudioPort_ == 0
 #ifdef RING_VIDEO
-        || getLocalVideoPort() == 0
+        || localVideoPort_ == 0
 #endif
         )
         generateMediaPorts();
@@ -154,19 +155,19 @@ SIPCall::generateMediaPorts()
     // Reference: http://www.cs.columbia.edu/~hgs/rtp/faq.html#ports
     // We only want to set ports to new values if they haven't been set
     const unsigned callLocalAudioPort = account.generateAudioPort();
-    if (getLocalAudioPort() != 0)
-        account.releasePort(getLocalAudioPort());
-    setLocalAudioPort(callLocalAudioPort);
+    if (localAudioPort_ != 0)
+        account.releasePort(localAudioPort_);
+    localAudioPort_ = callLocalAudioPort;
     sdp_->setLocalPublishedAudioPort(callLocalAudioPort);
 
 #ifdef RING_VIDEO
     // https://projects.savoirfairelinux.com/issues/17498
     const unsigned int callLocalVideoPort = account.generateVideoPort();
-    if (getLocalVideoPort() != 0)
-        account.releasePort(getLocalVideoPort());
+    if (localVideoPort_ != 0)
+        account.releasePort(localVideoPort_);
     // this should already be guaranteed by SIPAccount
-    assert(getLocalAudioPort() != callLocalVideoPort);
-    setLocalVideoPort(callLocalVideoPort);
+    assert(localAudioPort_ != callLocalVideoPort);
+    localVideoPort_ = callLocalVideoPort;
     sdp_->setLocalPublishedVideoPort(callLocalVideoPort);
 #endif
 }
@@ -234,7 +235,7 @@ SIPCall::SIPSessionReinvite()
                               getState() == CallState::HOLD))
         return !PJ_SUCCESS;
 
-    if (initIceTransport(true))
+    if (initIceMediaTransport(true))
         setupLocalSDPFromIce();
 
     pjsip_tx_data* tdata;
@@ -665,9 +666,9 @@ SIPCall::sendTextMessage(const std::map<std::string, std::string>& messages,
     //TODO: for now we ignore the "from" (the previous implementation for sending this info was
     //      buggy and verbose), another way to send the original message sender will be implemented
     //      in the future
-    if (not subcalls.empty()) {
+    if (not subcalls_.empty()) {
         pendingOutMessages_.emplace_back(messages, from);
-        for (auto& c : subcalls)
+        for (auto& c : subcalls_)
             c->sendTextMessage(messages, from);
     } else {
         if (inv) {
@@ -686,6 +687,7 @@ SIPCall::removeCall()
 {
     RING_WARN("[call:%s] removeCall()", getCallId().c_str());
     Call::removeCall();
+    mediaTransport_.reset();
     inv.reset();
     setTransport({});
 }
@@ -712,7 +714,7 @@ SIPCall::onAnswered()
     RING_WARN("[call:%s] onAnswered()", getCallId().c_str());
     if (getConnectionState() != ConnectionState::CONNECTED) {
         setState(CallState::ACTIVE, ConnectionState::CONNECTED);
-        if (not quiet)
+        if (not isSubcall())
             Manager::instance().peerAnsweredCall(*this);
     }
 }
@@ -721,31 +723,38 @@ void
 SIPCall::onPeerRinging()
 {
     setState(ConnectionState::RINGING);
-    Manager::instance().peerRingingCall(*this);
 }
 
 void
 SIPCall::setupLocalSDPFromIce()
 {
-    if (not iceTransport_) {
-        RING_WARN("[call:%s] null icetransport, no attributes added to SDP", getCallId().c_str());
+    auto media_tr = getIceMediaTransport();
+
+    if (not media_tr) {
+        RING_WARN("[call:%s] no media ICE transport, SDP not changed", getCallId().c_str());
         return;
     }
 
     // we need an initialized ICE to progress further
-    if (iceTransport_->waitForInitialization(DEFAULT_ICE_INIT_TIMEOUT) <= 0) {
+    if (media_tr->waitForInitialization(DEFAULT_ICE_INIT_TIMEOUT) <= 0) {
         RING_ERR("[call:%s] Medias' ICE init failed", getCallId().c_str());
         return;
     }
 
-    sdp_->addIceAttributes(iceTransport_->getLocalAttributes());
+    if (const auto& ip = getSIPAccount().getPublishedIpAddress()) {
+        for (unsigned compId = 1; compId <= media_tr->getComponentCount(); ++compId)
+            media_tr->registerPublicIP(compId, ip);
+    }
+
+    RING_WARN("[call:%s] fill SDP with ICE transport %p", getCallId().c_str(), media_tr);
+    sdp_->addIceAttributes(media_tr->getLocalAttributes());
 
     // Add video and audio channels
-    sdp_->addIceCandidates(SDP_AUDIO_MEDIA_ID, iceTransport_->getLocalCandidates(ICE_AUDIO_RTP_COMPID));
-    sdp_->addIceCandidates(SDP_AUDIO_MEDIA_ID, iceTransport_->getLocalCandidates(ICE_AUDIO_RTCP_COMPID));
+    sdp_->addIceCandidates(SDP_AUDIO_MEDIA_ID, media_tr->getLocalCandidates(ICE_AUDIO_RTP_COMPID));
+    sdp_->addIceCandidates(SDP_AUDIO_MEDIA_ID, media_tr->getLocalCandidates(ICE_AUDIO_RTCP_COMPID));
 #ifdef RING_VIDEO
-    sdp_->addIceCandidates(SDP_VIDEO_MEDIA_ID, iceTransport_->getLocalCandidates(ICE_VIDEO_RTP_COMPID));
-    sdp_->addIceCandidates(SDP_VIDEO_MEDIA_ID, iceTransport_->getLocalCandidates(ICE_VIDEO_RTCP_COMPID));
+    sdp_->addIceCandidates(SDP_VIDEO_MEDIA_ID, media_tr->getLocalCandidates(ICE_VIDEO_RTP_COMPID));
+    sdp_->addIceCandidates(SDP_VIDEO_MEDIA_ID, media_tr->getLocalCandidates(ICE_VIDEO_RTCP_COMPID));
 #endif
 }
 
@@ -753,12 +762,13 @@ std::vector<IceCandidate>
 SIPCall::getAllRemoteCandidates()
 {
     std::vector<IceCandidate> rem_candidates;
+    auto media_tr = getIceMediaTransport();
 
-    auto addSDPCandidates = [this](unsigned sdpMediaId,
-                                   std::vector<IceCandidate>& out) {
+    auto addSDPCandidates = [&, this](unsigned sdpMediaId,
+                                      std::vector<IceCandidate>& out) {
         IceCandidate cand;
         for (auto& line : sdp_->getIceCandidates(sdpMediaId)) {
-            if (iceTransport_->getCandidateFromSDP(line, cand)) {
+            if (media_tr->getCandidateFromSDP(line, cand)) {
                 RING_DBG("[call:%s] add remote ICE candidate: %s", getCallId().c_str(), line.c_str());
                 out.emplace_back(cand);
             }
@@ -773,23 +783,6 @@ SIPCall::getAllRemoteCandidates()
     return rem_candidates;
 }
 
-bool
-SIPCall::startIce()
-{
-    if (not iceTransport_ or iceTransport_->isFailed() or not iceTransport_->isInitialized())
-        return false;
-    if (iceTransport_->isStarted()) {
-        RING_DBG("[call:%s] ICE already started", getCallId().c_str());
-        return true;
-    }
-    auto rem_ice_attrs = sdp_->getIceAttributes();
-    if (rem_ice_attrs.ufrag.empty() or rem_ice_attrs.pwd.empty()) {
-        RING_ERR("[call:%s] ICE empty attributes", getCallId().c_str());
-        return false;
-    }
-    return iceTransport_->start(rem_ice_attrs, getAllRemoteCandidates());
-}
-
 bool
 SIPCall::useVideoCodec(const AccountVideoCodecInfo* codec) const
 {
@@ -856,11 +849,15 @@ SIPCall::startAllMedia()
             continue;
         }
 
+        auto new_mtu = transport_->getTlsMtu();
+        avformatrtp_->setMtu(new_mtu);
+
 #ifdef RING_VIDEO
         if (local.type == MEDIA_VIDEO)
             videortp_->switchInput(videoInput_);
-#endif
 
+        videortp_->setMtu(new_mtu);
+#endif
         rtp->updateMedia(remote, local);
 
         // Not restarting media loop on hold as it's a huge waste of CPU ressources
@@ -871,7 +868,7 @@ SIPCall::startAllMedia()
                            newIceSocket(ice_comp_id + 1));
                 ice_comp_id += 2;
             } else
-                rtp->start();
+                rtp->start(nullptr, nullptr);
         }
 
         switch (local.type) {
@@ -887,7 +884,7 @@ SIPCall::startAllMedia()
         }
     }
 
-    if (not quiet and peerHolding_ != peer_holding) {
+    if (not isSubcall() and peerHolding_ != peer_holding) {
         peerHolding_ = peer_holding;
         emitSignal<DRing::CallSignal::PeerHold>(getCallId(), peerHolding_);
     }
@@ -933,7 +930,7 @@ SIPCall::muteMedia(const std::string& mediaType, bool mute)
         isVideoMuted_ = mute;
         videoInput_ = isVideoMuted_ ? "" : Manager::instance().getVideoManager().videoDeviceMonitor.getMRLForDefaultDevice();
         DRing::switchInput(getCallId(), videoInput_);
-        if (not quiet)
+        if (not isSubcall())
             emitSignal<DRing::CallSignal::VideoMuted>(getCallId(), isVideoMuted_);
 #endif
     } else if (mediaType.compare(DRing::Media::Details::MEDIA_TYPE_AUDIO) == 0) {
@@ -941,48 +938,94 @@ SIPCall::muteMedia(const std::string& mediaType, bool mute)
         RING_WARN("[call:%s] audio muting %s", getCallId().c_str(), bool_to_str(mute));
         isAudioMuted_ = mute;
         avformatrtp_->setMuted(isAudioMuted_);
-        if (not quiet)
+        if (not isSubcall())
             emitSignal<DRing::CallSignal::AudioMuted>(getCallId(), isAudioMuted_);
     }
 }
 
+/// \brief Prepare media transport and launch media stream based on negotiated SDP
+///
+/// This method has to be called by link (ie SipVoIpLink) when SDP is negotiated and
+/// media streams structures are knows.
+/// In case of ICE transport used, the medias streams are launched asynchonously when
+/// the transport is negotiated.
 void
 SIPCall::onMediaUpdate()
 {
-    RING_WARN("[call:%s] onMediaUpdate", getCallId().c_str());
-    stopAllMedia();
-    openPortsUPnP();
-    if (startIce()) {
-        if (not quiet)
-            waitForIceAndStartMedia();
-    } else {
-        RING_WARN("[call:%s] ICE not used for media", getCallId().c_str());
+    RING_WARN("[call:%s] medias changed", getCallId().c_str());
+
+    // If ICE is not used, start medias now
+    auto rem_ice_attrs = sdp_->getIceAttributes();
+    if (rem_ice_attrs.ufrag.empty() or rem_ice_attrs.pwd.empty()) {
+        RING_WARN("[call:%s] no remote ICE for medias", getCallId().c_str());
+        stopAllMedia();
         startAllMedia();
+        return;
     }
+
+    // Main call (no subcalls) must wait for ICE now, the rest of code needs to access
+    // to a negotiated transport.
+    if (not isSubcall())
+        waitForIceAndStartMedia();
 }
 
 void
 SIPCall::waitForIceAndStartMedia()
 {
-    auto ice = iceTransport_;
-    auto iceTimeout = std::chrono::steady_clock::now() + std::chrono::seconds(10);
-    auto wthis = std::weak_ptr<SIPCall>(std::static_pointer_cast<SIPCall>(shared_from_this()));
-    Manager::instance().addTask([wthis,ice,iceTimeout] {
-        if (auto sthis = wthis.lock()) {
-            auto& this_ = *sthis;
-            if (ice != this_.iceTransport_) {
-                RING_WARN("[call:%s] ICE transport replaced", this_.getCallId().c_str());
+    // Initialization waiting task
+    auto weak_call = std::weak_ptr<SIPCall>(std::static_pointer_cast<SIPCall>(shared_from_this()));
+    Manager::instance().addTask([weak_call] {
+        // TODO: polling algo, to it by event
+        if (auto call = weak_call.lock()) {
+            auto ice = call->getIceMediaTransport();
+
+            if (ice->isFailed()) {
+                RING_ERR("[call:%s] Media ICE init failed", call->getCallId().c_str());
+                call->onFailure(EIO);
                 return false;
             }
-            /* First step: wait for an ICE transport for SIP channel */
-            if (this_.iceTransport_->isFailed() or std::chrono::steady_clock::now() >= iceTimeout) {
-                RING_DBG("[call:%s] ICE init failed (or timeout)", this_.getCallId().c_str());
-                this_.onFailure(ETIMEDOUT);
-                return false;
-            }
-            if (not this_.iceTransport_->isRunning())
+
+            if (!ice->isInitialized())
                 return true;
-            this_.startAllMedia();
+
+            // Start transport on SDP data and wait for negotation
+            auto rem_ice_attrs = call->sdp_->getIceAttributes();
+            if (rem_ice_attrs.ufrag.empty() or rem_ice_attrs.pwd.empty()) {
+                RING_ERR("[call:%s] Media ICE attributes empty", call->getCallId().c_str());
+                call->onFailure(EIO);
+                return false;
+            }
+            if (not ice->start(rem_ice_attrs, call->getAllRemoteCandidates())) {
+                RING_ERR("[call:%s] Media ICE start failed", call->getCallId().c_str());
+                call->onFailure(EIO);
+                return false;
+            }
+
+            // Negotiation waiting task
+            Manager::instance().addTask([weak_call] {
+                if (auto call = weak_call.lock()) {
+                    auto ice = call->getIceMediaTransport();
+
+                    if (ice->isFailed()) {
+                        RING_ERR("[call:%s] Media ICE negotation failed", call->getCallId().c_str());
+                        call->onFailure(EIO);
+                        return false;
+                    }
+
+                    if (not ice->isRunning())
+                        return true;
+
+                    // Nego succeed: move to the new media transport
+                    call->stopAllMedia();
+                    if (call->tmpMediaTransport_)
+                        call->mediaTransport_ = std::move(call->tmpMediaTransport_);
+                    call->startAllMedia();
+                    return false;
+                }
+                return false;
+            });
+
+            return false;
         }
         return false;
     });
@@ -997,15 +1040,11 @@ SIPCall::onReceiveOffer(const pjmedia_sdp_session* offer)
         acc.getActiveAccountCodecInfoList(MEDIA_AUDIO),
         acc.getActiveAccountCodecInfoList(acc.isVideoEnabled() ? MEDIA_VIDEO : MEDIA_NONE),
         acc.getSrtpKeyExchange(),
-        getState() == CallState::HOLD
-    );
-    auto ice_attrs = Sdp::getIceAttributes(offer);
-    if (not ice_attrs.ufrag.empty() and not ice_attrs.pwd.empty()) {
-        if (initIceTransport(false))
-            setupLocalSDPFromIce();
-    }
+        getState() == CallState::HOLD);
+    setRemoteSdp(offer);
     sdp_->startNegotiation();
     pjsip_inv_set_sdp_answer(inv.get(), sdp_->getLocalSdpSession());
+    openPortsUPnP();
 }
 
 void
@@ -1101,37 +1140,81 @@ SIPCall::InvSessionDeleter::operator ()(pjsip_inv_session* inv) const noexcept
     // prevent this from getting accessed in callbacks
     // RING_WARN: this is not thread-safe!
     inv->mod_data[getSIPVoIPLink()->getModId()] = nullptr;
-    pjsip_dlg_dec_lock(inv->dlg);
 }
 
 bool
-SIPCall::initIceTransport(bool master, unsigned channel_num)
+SIPCall::initIceMediaTransport(bool master, unsigned channel_num)
 {
-    auto result = Call::initIceTransport(master, channel_num);
-    if (result) {
-        if (const auto& publicIP = getSIPAccount().getPublishedIpAddress()) {
-            for (unsigned compId = 1; compId <= iceTransport_->getComponentCount(); ++compId)
-                iceTransport_->registerPublicIP(compId, publicIP);
-        }
-    }
-    return result;
+    RING_DBG("[call:%s] create media ICE transport", getCallId().c_str());
+
+    auto& iceTransportFactory = Manager::instance().getIceTransportFactory();
+    tmpMediaTransport_ = iceTransportFactory.createTransport(getCallId().c_str(),
+                                                             channel_num, master,
+                                                             getAccount().getIceOptions());
+    return static_cast<bool>(tmpMediaTransport_);
 }
 
 void
-SIPCall::merge(const std::shared_ptr<SIPCall>& scall)
+SIPCall::merge(Call& call)
 {
-    RING_WARN("SIPCall::merge %s -> %s", scall->getCallId().c_str(), getCallId().c_str());
-    inv = std::move(scall->inv);
+    RING_DBG("[sipcall:%s] merge subcall %s", getCallId().c_str(), call.getCallId().c_str());
+
+    // This static cast is safe as this method is private and overload Call::merge
+    auto& subcall = static_cast<SIPCall&>(call);
+
+    inv = std::move(subcall.inv);
     inv->mod_data[getSIPVoIPLink()->getModId()] = this;
-    setTransport(scall->transport_);
-    sdp_ = std::move(scall->sdp_);
-    peerHolding_ = scall->peerHolding_;
-    upnp_ = std::move(scall->upnp_);
-    std::copy_n(scall->contactBuffer_, PJSIP_MAX_URL_SIZE, contactBuffer_);
-    pj_strcpy(&contactHeader_, &scall->contactHeader_);
-    Call::merge(scall);
-    if (iceTransport_->isStarted())
-        waitForIceAndStartMedia();
+    setTransport(subcall.transport_);
+    sdp_ = std::move(subcall.sdp_);
+    peerHolding_ = subcall.peerHolding_;
+    upnp_ = std::move(subcall.upnp_);
+    std::copy_n(subcall.contactBuffer_, PJSIP_MAX_URL_SIZE, contactBuffer_);
+    pj_strcpy(&contactHeader_, &subcall.contactHeader_);
+    localAudioPort_ = subcall.localAudioPort_;
+    localVideoPort_ = subcall.localVideoPort_;
+    mediaTransport_ = std::move(subcall.mediaTransport_);
+    tmpMediaTransport_ = std::move(subcall.tmpMediaTransport_);
+
+    Call::merge(subcall);
+
+    waitForIceAndStartMedia();
+}
+
+void
+SIPCall::setRemoteSdp(const pjmedia_sdp_session* sdp)
+{
+    if (!sdp)
+        return;
+
+    // If ICE is not used, start medias now
+    auto rem_ice_attrs = sdp_->getIceAttributes();
+    if (rem_ice_attrs.ufrag.empty() or rem_ice_attrs.pwd.empty()) {
+        RING_WARN("[call:%s] no ICE data in remote SDP", getCallId().c_str());
+        return;
+    }
+
+    if (not initIceMediaTransport(false)) {
+        // Fatal condition
+        // TODO: what's SIP rfc says about that?
+        // (same question in waitForIceAndStartMedia)
+        onFailure(EIO);
+        return;
+    }
+
+    // WARNING: This call blocks! (need ice init done)
+    setupLocalSDPFromIce();
+}
+
+bool
+SIPCall::isIceRunning() const
+{
+    return mediaTransport_ and mediaTransport_->isRunning();
+}
+
+std::unique_ptr<IceSocket>
+SIPCall::newIceSocket(unsigned compId)
+{
+    return std::unique_ptr<IceSocket> {new IceSocket(mediaTransport_, compId)};
 }
 
 } // namespace ring
